#include <dirent.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <xmmsctrl.h>

DIR *directory;
struct dirent *dentry;
struct dirent *newest_file;
struct stat filestat;
struct stat *filestatp = &filestat;
int havefile = 0;
char *usernamep;
size_t usernamel;
struct passwd userinfo;
struct passwd *userinfop = &userinfo;
time_t newest = 0;
char username[256];
char *session;

void help(void) {
	fprintf(stdout, "The follwing commands are possible:\n\n"
	"get_version,\n"
	"get_ouput_time,\n"
	"play,\n"
	"pause,\n"
	"stop,\n"
	"quit,\n"
	"playlist_prev,\n"
	"playlist_next,\n"
	"get_playlist_pos,\n"
	"get_playlist_length,\n"
	"get_playlist_time trackno,\n"
	"get_playlist_title trackno,\n"
	"is_repeat,\n"
	"toggle_repeat,\n"
	"is_shuffle,\n"
	"toggle_shuffle,\n"
	"get_main_volume,\n"
	"set_main_volume newvolume,\n"
	"inc_main_volume increment,\n"
	"dec_main_volume decrement.\n\n");
}

int main(int argc, char *argv[]){
	if (argc < 2) {
		fprintf(stderr, "Please start with at least 1 parameter.\n\n");
		help();
		exit(2);
	}
	directory = opendir("/tmp");
	if (directory == NULL){
		fprintf(stderr, "Can't open /tmp directory.\n");
		exit(1);
	}
	if (chdir("/tmp") == -1) {
		fprintf(stderr, "Can't change to /tmp directory.\n");
        exit(1);
	}
	while ((dentry = readdir(directory))) {
		if ((strstr(dentry->d_name, "xmms")) != NULL) {
			stat(dentry->d_name, filestatp);
			if (filestat.st_atime > newest) {
				newest = filestat.st_atime;
				newest_file = dentry;
				havefile = 1;
			}
		}
	}
	if (! havefile) {
		fprintf(stderr, "Can't find the commandsocket of xmms.\n");
        exit(1);
	}
	usernamep = (strchr(newest_file->d_name, '_'))+1;
	usernamel = strcspn(usernamep, ".");
	if (usernamel > 254) {
        fprintf(stderr, "Username too long.\n");
        exit(3);
    }
	strncpy(username, usernamep, usernamel);
	session = strchr(newest_file->d_name, '.')+1;
	*session -= 48;
	//fprintf(stderr, "%d\n", *session);
	userinfop = getpwnam(username);
	setuid(geteuid());
	setuid(userinfop->pw_uid);
	seteuid(userinfop->pw_uid);
	if (strlen(argv[1]) > 255) {
        fprintf(stderr, "Parameter too long.\n");
        exit(3);
    }		
	//fprintf(stderr, "%d\n", userinfop->pw_uid);
	closedir(directory);
	// Ab hier werden nur noch die xxms_* Funktionen ausgefhrt.
	if (strstr (argv[1], "help" ) != NULL) {
		help();
		exit(0);
	}
	if (strstr (argv[1], "get_version" ) != NULL) {
		fprintf(stdout, "%d\n", xmms_remote_get_version(*session));
		exit(0);
	}
	if (strstr (argv[1], "get_output_time" ) != NULL) {
		fprintf(stdout, "%d\n", xmms_remote_get_output_time(*session));
		exit(0);
	}
	if (strstr (argv[1], "playlist_prev" ) != NULL) {
		xmms_remote_playlist_prev(*session);
		exit(0);
	}
	if (strstr (argv[1], "playlist_next" ) != NULL) {
		xmms_remote_playlist_next(*session);
		exit(0);
	}
	if (strstr (argv[1], "get_playlist_pos" ) != NULL) {
		fprintf(stdout, "%d\n", xmms_remote_get_playlist_pos(*session)+1);
		exit(0);
	}
	if (strstr (argv[1], "get_playlist_length" ) != NULL) {
		fprintf(stdout, "%d\n", xmms_remote_get_playlist_length(*session));
		exit(0);
	}
	if (strstr (argv[1], "get_playlist_time" ) != NULL) {
		if (argc < 3) {
		fprintf(stderr, "Please supply the tracknumber.\n");
		exit(2);
		} else {
			int track = 0;
			track = atoi(argv[2]);
			fprintf(stdout, "%d\n", xmms_remote_get_playlist_time(*session, track-1));
			exit(0);
		}
	}
	if (strstr (argv[1], "get_playlist_title" ) != NULL) {
		if (argc < 3) {
		fprintf(stderr, "Please supply the tracknumber.\n");
		exit(2);
		} else {
			int track = 0;
			track = atoi(argv[2]);
			fprintf(stdout, "%s\n", xmms_remote_get_playlist_title(*session, track-1));
			exit(0);
		}
	}
	if (strstr (argv[1], "get_main_volume" ) != NULL) {
		fprintf(stdout, "%d\n", xmms_remote_get_main_volume(*session));
		exit(0);
	}
	if (strstr (argv[1], "set_main_volume" ) != NULL) {
		if (argc < 3) {
		fprintf(stderr, "Please supply the Volume.\n");
		exit(2);
		} else {
			int volume = 0;
			volume = strtoul(argv[2], NULL, 10);
			xmms_remote_set_main_volume(*session, volume);
		}
		exit(0);
	}
	if (strstr (argv[1], "inc_main_volume" ) != NULL) {
		if (argc < 3) {
		fprintf(stderr, "Please supply the increment.\n");
		exit(2);
		} else {
			int volume = 0;
			volume = xmms_remote_get_main_volume(*session);
			volume = volume + strtoul(argv[2], NULL, 10) + 1;
			if ( volume > 100 ) {
				volume = 100;
			}
			xmms_remote_set_main_volume(*session, volume);
		}
		exit(0);
	}
	if (strstr (argv[1], "dec_main_volume" ) != NULL) {
		if (argc < 3) {
		fprintf(stderr, "Please supply the decrement.\n");
		exit(2);
		} else {
			int volume = 0;
			volume = xmms_remote_get_main_volume(*session);
			volume = volume - strtoul(argv[2], NULL, 10) - 1;
			if ( volume < 1 ) {
				volume = 0;
			}
			xmms_remote_set_main_volume(*session, volume);
		}
		exit(0);
	}
	if (strstr (argv[1], "is_repeat" ) != NULL) {
		fprintf(stdout, "%d\n", xmms_remote_is_repeat(*session));
		exit(0);
	}
	if (strstr (argv[1], "toggle_repeat" ) != NULL) {
		xmms_remote_toggle_repeat(*session);
		exit(0);
	}
	if (strstr (argv[1], "is_shuffle" ) != NULL) {
		fprintf(stdout, "%d\n", xmms_remote_is_shuffle(*session));
		exit(0);
	}
	if (strstr (argv[1], "toggle_shuffle" ) != NULL) {
		xmms_remote_toggle_shuffle(*session);
		exit(0);
	}
	if (strstr (argv[1], "play" ) != NULL) {
		xmms_remote_play(*session);
		exit(0);
	}
	if (strstr (argv[1], "pause" ) != NULL) {
		xmms_remote_pause(*session);
		exit(0);
	}
	if (strstr (argv[1], "stop" ) != NULL) {
		xmms_remote_stop(*session);
		exit(0);
	}
	if (strstr (argv[1], "quit" ) != NULL) {
		xmms_remote_quit(*session);
		exit(0);
	}
	return(0);
}
